@@ -32,12 +32,12 @@ LOG = logging.getLogger(__name__)  # not used at the moment
 def get_lcmocc(context, lcmocc_id):
     lcmocc = objects.VnfLcmOpOccV2.get_by_id(context, lcmocc_id)
     if lcmocc is None:
-        raise sol_ex.VnfLcmOpOccNotFound(lcmocc_id=id)
+        raise sol_ex.VnfLcmOpOccNotFound(lcmocc_id=lcmocc_id)
     return lcmocc
 
 
-def get_lcmocc_all(context):
-    return objects.VnfLcmOpOccV2.get_all(context)
+def get_lcmocc_all(context, marker=None):
+    return objects.VnfLcmOpOccV2.get_all(context, marker)
 
 
 def lcmocc_href(lcmocc_id, endpoint):
@@ -97,7 +97,10 @@ def make_lcmocc_notif_data(subsc, lcmocc, endpoint):
     else:
         notif_data.notificationStatus = 'RESULT'
 
-    if lcmocc.obj_attr_is_set('error'):
+    if lcmocc.operationState in [fields.LcmOperationStateType.FAILED_TEMP,
+                                 fields.LcmOperationStateType.FAILED,
+                                 fields.LcmOperationStateType.ROLLED_BACK]:
+        # The existence of lcmocc.error is guaranteed.
         notif_data.error = lcmocc.error
 
     if notif_data.verbosity == fields.LcmOpOccNotificationVerbosityType.FULL:
@@ -126,6 +129,8 @@ def _make_affected_vnfc(vnfc, change_type, strgs):
         changeType=change_type,
         computeResource=vnfc.computeResource
     )
+    if vnfc.obj_attr_is_set('zoneId'):
+        affected_vnfc.zoneId = vnfc.zoneId
     if vnfc.obj_attr_is_set('metadata'):
         affected_vnfc.metadata = vnfc.metadata
     if vnfc.obj_attr_is_set('vnfcCpInfo'):
@@ -147,6 +152,31 @@ def _make_affected_vnfc(vnfc, change_type, strgs):
     return affected_vnfc
 
 
+def _make_affected_vnfc_modified(vnfc, vnfc_saved):
+    affected_vnfc = objects.AffectedVnfcV2(
+        id=vnfc.id,
+        vduId=vnfc.vduId,
+        changeType='MODIFIED',
+        computeResource=vnfc.computeResource
+    )
+    affected_vnfc.metadata = vnfc.metadata
+    # NOTE: cps may not be affected.
+    if vnfc.obj_attr_is_set('vnfcCpInfo'):
+        cp_ids = [cp.id for cp in vnfc.vnfcCpInfo]
+        affected_vnfc.affectedVnfcCpIds = cp_ids
+    if vnfc.obj_attr_is_set('storageResourceIds'):
+        added_str_ids = (set(vnfc.storageResourceIds) -
+                         set(vnfc_saved.storageResourceIds))
+        removed_str_ids = (set(vnfc_saved.storageResourceIds) -
+                           set(vnfc.storageResourceIds))
+        if added_str_ids:
+            affected_vnfc.addedStorageResourceIds = list(added_str_ids)
+        if removed_str_ids:
+            affected_vnfc.removedStorageResourceIds = list(removed_str_ids)
+
+    return affected_vnfc
+
+
 def _make_affected_vl(vl, change_type):
     affected_vl = objects.AffectedVirtualLinkV2(
         id=vl.id,
@@ -270,21 +300,24 @@ def _check_modification(inst_saved, inst, attr):
     return False
 
 
+def _check_modification_inst(obj, inst_saved, inst):
+    obj.vnfdId = inst.vnfdId
+    if inst_saved.vnfProvider != inst.vnfProvider:
+        obj.vnfProvider = inst.vnfProvider
+    if inst_saved.vnfProductName != inst.vnfProductName:
+        obj.vnfProductName = inst.vnfProductName
+    if inst_saved.vnfSoftwareVersion != inst.vnfSoftwareVersion:
+        obj.vnfSoftwareVersion = inst.vnfSoftwareVersion
+    if inst_saved.vnfdVersion != inst.vnfdVersion:
+        obj.vnfdVersion = inst.vnfdVersion
+
+
 def _change_vnf_info(lcmocc, inst_saved, inst):
     vnf_info_modify = objects.VnfInfoModificationsV2()
 
     # vnfdid is required, don't check the existence of the value.
     if inst_saved.vnfdId != inst.vnfdId:
-        vnf_info_modify.vnfdId = inst.vnfdId
-
-        if inst_saved.vnfProvider != inst.vnfProvider:
-            vnf_info_modify.vnfProvider = inst.vnfProvider
-        if inst_saved.vnfProductName != inst.vnfProductName:
-            vnf_info_modify.vnfProductName = inst.vnfProductName
-        if inst_saved.vnfSoftwareVersion != inst.vnfSoftwareVersion:
-            vnf_info_modify.vnfSoftwareVersion = inst.vnfSoftwareVersion
-        if inst_saved.vnfdVersion != inst.vnfdVersion:
-            vnf_info_modify.vnfdVersion = inst.vnfdVersion
+        _check_modification_inst(vnf_info_modify, inst_saved, inst)
 
     attrs = ['vnfInstanceName', 'vnfInstanceDescription',
              'vnfConfigurableProperties', 'metadata', 'extensions']
@@ -335,6 +368,18 @@ def _change_vnf_info(lcmocc, inst_saved, inst):
     lcmocc.changedInfo = vnf_info_modify
 
 
+def _create_modifications_triggered_by_vnfpkg_change(lcmocc, inst_saved, inst):
+    vnfpkg_modifications = objects.ModificationsTriggeredByVnfPkgChangeV2()
+    _check_modification_inst(vnfpkg_modifications, inst_saved, inst)
+
+    attrs = ['vnfConfigurableProperties', 'metadata', 'extensions']
+    for attr in attrs:
+        if _check_modification(inst_saved, inst, attr):
+            setattr(vnfpkg_modifications, attr, getattr(inst, attr))
+
+    lcmocc.modificationsTriggeredByVnfPkgChange = vnfpkg_modifications
+
+
 def update_lcmocc(lcmocc, inst_saved, inst):
     # if operation is MODIFY_INFO, make changedInfo of lcmocc.
     # for other operations, make ResourceChanges of lcmocc
@@ -383,15 +428,7 @@ def update_lcmocc(lcmocc, inst_saved, inst):
                            for strg in inst_info.virtualStorageResourceInfo
                            if strg.id in added_strgs]
 
-    removed_vnfcs, added_vnfcs, common_objs = _calc_diff('vnfcResourceInfo')
-    updated_vnfcs = []
-    if lcmocc.operation == fields.LcmOperationType.CHANGE_VNFPKG:
-        updated_vnfcs = [
-            obj.id for obj in inst_info.vnfcResourceInfo
-            if obj.metadata.get('current_vnfd_id') != inst_saved.vnfdId
-            and obj.id in common_objs and
-            obj.metadata.get('current_vnfd_id') is not None]
-
+    removed_vnfcs, added_vnfcs, common_vnfcs = _calc_diff('vnfcResourceInfo')
     affected_vnfcs = []
     if removed_vnfcs:
         affected_vnfcs += [
@@ -405,11 +442,21 @@ def update_lcmocc(lcmocc, inst_saved, inst):
             for vnfc in inst_info.vnfcResourceInfo
             if vnfc.id in added_vnfcs
         ]
+    if lcmocc.operation == fields.LcmOperationType.CHANGE_VNFPKG:
+        def _get_vnfc(vnfc_id, inst_info):
+            for vnfc in inst_info.vnfcResourceInfo:
+                if vnfc.id == vnfc_id:
+                    return vnfc
 
-    if updated_vnfcs:
-        affected_vnfcs += [_make_affected_vnfc(vnfc, 'MODIFIED', added_strgs)
-                           for vnfc in inst_info.vnfcResourceInfo
-                           if vnfc.id in updated_vnfcs]
+        for obj_id in common_vnfcs:
+            vnfc = _get_vnfc(obj_id, inst_info)
+            vnfc_saved = _get_vnfc(obj_id, inst_saved.instantiatedVnfInfo)
+            if vnfc.metadata != vnfc_saved.metadata:
+                affected_vnfcs.append(
+                    _make_affected_vnfc_modified(vnfc, vnfc_saved))
+
+        _create_modifications_triggered_by_vnfpkg_change(
+            lcmocc, inst_saved, inst)
 
     removed_vls, added_vls, common_vls = _calc_diff(
         'vnfVirtualLinkResourceInfo')
@@ -531,6 +578,24 @@ def get_grant_req_and_grant(context, lcmocc):
     return grant_reqs[0], grant
 
 
+def is_lcmocc_failure_status(context, inst_id):
+    inst_lcmoccs = objects.VnfLcmOpOccV2.get_by_filter(
+        context, vnfInstanceId=inst_id)
+    failed_temp_lcmoccs = [
+        lcmocc for lcmocc in inst_lcmoccs
+        if lcmocc.operationState == fields.LcmOperationStateType.FAILED_TEMP]
+    failed_lcmocc = [
+        latest_lcmocc for latest_lcmocc in inst_lcmoccs
+        if latest_lcmocc.startTime == max(
+            [lcmocc.startTime for lcmocc in inst_lcmoccs])]
+
+    if failed_temp_lcmoccs or (failed_lcmocc[0].operationState ==
+                               fields.LcmOperationStateType.FAILED):
+        return True
+
+    return False
+
+
 def check_lcmocc_in_progress(context, inst_id):
     # if the controller or conductor executes an operation for the vnf
     # instance (i.e. operationState is ...ING), other operation for
@@ -542,3 +607,8 @@ def check_lcmocc_in_progress(context, inst_id):
         operationState=fields.LcmOperationStateType.FAILED_TEMP)
     if lcmoccs:
         raise sol_ex.OtherOperationInProgress(inst_id=inst_id)
+
+
+def update_lcmocc_status(lcmocc, op_state):
+    lcmocc.operationState = op_state
+    lcmocc.stateEnteredTime = datetime.utcnow()
